"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
class AbsApiClass {
    constructor(type) {
        this.typeMap = {
            'apiSuccessClass': 'apiSuccess',
            'apiParamClass': 'apiParam'
        };
        this.ast = new ts_simple_ast_1.Project();
        this.type = type;
    }
    parseElements(elements, element) {
        elements.pop();
        this.extractElementContent(element.content);
        const classAst = this.getClassAst({ filePath: this.filePath, className: this.className });
        if (!classAst) {
            return elements;
        }
        const newElements = this.transferToNewElement({ filePath: this.filePath, classAst, prefix: '' });
        elements.push(...newElements);
    }
    transferToNewElement({ filePath, classAst, prefix }) {
        const properties = classAst.getProperties();
        let elements = [];
        for (const property of properties) {
            const name = prefix ? `${prefix}.${property.getName()}` : property.getName();
            const node = property.getJsDocs()[0];
            const comment = node ? node.getComment() : property.getName();
            const description = prefix ? `${prefix} > ${comment}` : comment;
            const type = property.getType().getText();
            const content = `{${this.capitalize(type)}} ${name} ${description}`;
            const transferType = this.typeMap[this.type];
            elements.push({
                content,
                name: transferType.toLowerCase(),
                source: `@${transferType} ${content}\n`,
                sourceName: transferType
            });
        }
        return elements;
    }
    extractElementContent(content) {
        let startPos = content.indexOf('(');
        let endPos = content.indexOf(')', startPos);
        this.filePath = content.substring(startPos + 1, endPos);
        startPos = content.indexOf('{', endPos);
        endPos = content.indexOf('}', startPos);
        this.className = content.substring(startPos + 1, endPos);
    }
    getClassAst({ filePath, className }) {
        const file = this.ast.addExistingSourceFile(filePath);
        return file.getClass(className);
    }
    capitalize(text) {
        return text.charAt(0).toUpperCase() + text.slice(1).toLowerCase();
    }
}
exports.AbsApiClass = AbsApiClass;
//# sourceMappingURL=AbsApiClass.js.map